import { retrieveOrGenerateKeyPair } from "../utils/crypto.ts";

const URL_INPUT_ID = "request-url";
const TEMPLATE = `
      <details open>
        <summary>How To Use:</summary>
        <ol>
        <li><strong>Request:</strong> Copy and share the URL below with the person you want to receive data from.</li>
        <li><strong>Send:</strong> The recipient will open the URL, enter their data, and hit 'Generate Response'. This generates a new URL with their encrypted message.</li>
        <li><strong>Receive:</strong> They send you the newly generated URL. Open it to view the decrypted message. Only your browser can decrypt it.</li>
        </ol>
      </details>

      <form>
        <input
          type="text"
          name="request-url"
          id="${URL_INPUT_ID}"
          placeholder=""
          aria-label="Request URL"
          required
        />
      </form>
    `;

export async function setupRequestPage(element: HTMLElement) {
  element.innerHTML = TEMPLATE;

  const url = await generateUrl();
  const input = document.getElementById(URL_INPUT_ID) as HTMLInputElement;
  input.value = url.toString();
}

async function generateUrl(): Promise<URL> {
  const { publicKey: ecdhPublic } = await retrieveOrGenerateKeyPair();

  // Generate URL with public key as the 'p' search parameter
  const ecdhPublicJwk = await window.crypto.subtle.exportKey("jwk", ecdhPublic);
  const url = new URL(window.location.toString());
  url.search = "";
  url.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}`;

  // Return the generated URL
  return url;
}
